package org.eclipse.viatra2.emf.incquery.tooling.generator.databinding;

import com.google.inject.Inject;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Map;
import java.util.Set;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.emf.common.util.EList;
import org.eclipse.pde.core.plugin.IPluginElement;
import org.eclipse.pde.core.plugin.IPluginExtension;
import org.eclipse.viatra2.emf.incquery.databinding.runtime.util.DatabindingAdapterUtil;
import org.eclipse.viatra2.emf.incquery.tooling.generator.ExtensionGenerator;
import org.eclipse.viatra2.emf.incquery.tooling.generator.fragments.IGenerationFragment;
import org.eclipse.viatra2.emf.incquery.tooling.generator.util.EMFPatternLanguageJvmModelInferrerUtil;
import org.eclipse.viatra2.patternlanguage.core.helper.CorePatternLanguageHelper;
import org.eclipse.viatra2.patternlanguage.core.patternLanguage.Annotation;
import org.eclipse.viatra2.patternlanguage.core.patternLanguage.AnnotationParameter;
import org.eclipse.viatra2.patternlanguage.core.patternLanguage.Pattern;
import org.eclipse.viatra2.patternlanguage.core.patternLanguage.ValueReference;
import org.eclipse.viatra2.patternlanguage.core.patternLanguage.impl.StringValueImpl;
import org.eclipse.xtend2.lib.StringConcatenation;
import org.eclipse.xtext.generator.IFileSystemAccess;
import org.eclipse.xtext.xbase.lib.CollectionLiterals;
import org.eclipse.xtext.xbase.lib.Conversions;
import org.eclipse.xtext.xbase.lib.Pair;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;
import org.eclipse.xtext.xbase.lib.StringExtensions;

@SuppressWarnings("all")
public class DatabindingGenerator implements IGenerationFragment {
  private static String DATABINDINGEXTENSION_PREFIX = "extension.databinding.";
  
  private static String DATABINDINGEXTENSION_POINT = "org.eclipse.viatra2.emf.incquery.databinding.runtime.databinding";
  
  @Inject
  private EMFPatternLanguageJvmModelInferrerUtil _eMFPatternLanguageJvmModelInferrerUtil;
  
  private static String annotationLiteral = "ObservableValue";
  
  public void generateFiles(final Pattern pattern, final IFileSystemAccess fsa) {
    boolean _hasAnnotationLiteral = this.hasAnnotationLiteral(pattern, DatabindingGenerator.annotationLiteral);
    if (_hasAnnotationLiteral) {
      String _packagePath = this._eMFPatternLanguageJvmModelInferrerUtil.getPackagePath(pattern);
      String _plus = (_packagePath + "/");
      String _name = pattern.getName();
      String _firstUpper = StringExtensions.toFirstUpper(_name);
      String _plus_1 = (_plus + _firstUpper);
      String _plus_2 = (_plus_1 + "DatabindingAdapter.java");
      CharSequence _patternHandler = this.patternHandler(pattern);
      fsa.generateFile(_plus_2, _patternHandler);
    }
  }
  
  public void cleanUp(final Pattern pattern, final IFileSystemAccess fsa) {
    String _packagePath = this._eMFPatternLanguageJvmModelInferrerUtil.getPackagePath(pattern);
    String _plus = (_packagePath + "/");
    String _realPatternName = this._eMFPatternLanguageJvmModelInferrerUtil.realPatternName(pattern);
    String _firstUpper = StringExtensions.toFirstUpper(_realPatternName);
    String _plus_1 = (_plus + _firstUpper);
    String _plus_2 = (_plus_1 + "DatabindingAdapter.java");
    fsa.deleteFile(_plus_2);
  }
  
  public Iterable<Pair<String,String>> removeExtension(final Pattern pattern) {
    String _databindingContributionId = this.databindingContributionId(pattern);
    Pair<String,String> _of = Pair.<String, String>of(_databindingContributionId, DatabindingGenerator.DATABINDINGEXTENSION_POINT);
    ArrayList<Pair<String,String>> _newArrayList = CollectionLiterals.<Pair<String,String>>newArrayList(_of);
    return _newArrayList;
  }
  
  public Collection<Pair<String,String>> getRemovableExtensions() {
    Pair<String,String> _of = Pair.<String, String>of(DatabindingGenerator.DATABINDINGEXTENSION_PREFIX, DatabindingGenerator.DATABINDINGEXTENSION_POINT);
    ArrayList<Pair<String,String>> _newArrayList = CollectionLiterals.<Pair<String,String>>newArrayList(_of);
    return _newArrayList;
  }
  
  public String[] getProjectDependencies() {
    ArrayList<String> _newArrayList = CollectionLiterals.<String>newArrayList("org.eclipse.core.databinding.property", 
      "org.eclipse.core.databinding.observable", 
      "org.eclipse.viatra2.emf.incquery.databinding.runtime", 
      "org.eclipse.viatra2.emf.incquery.runtime");
    return ((String[])Conversions.unwrapArray(_newArrayList, String.class));
  }
  
  public String getProjectPostfix() {
    return "databinding";
  }
  
  public String databindingContributionId(final Pattern pattern) {
    String _fullyQualifiedName = CorePatternLanguageHelper.getFullyQualifiedName(pattern);
    String _plus = (DatabindingGenerator.DATABINDINGEXTENSION_PREFIX + _fullyQualifiedName);
    return _plus;
  }
  
  public Iterable<IPluginExtension> extensionContribution(final Pattern pattern, final ExtensionGenerator exGen) {
    ArrayList<IPluginExtension> _xifexpression = null;
    boolean _hasAnnotationLiteral = this.hasAnnotationLiteral(pattern, DatabindingGenerator.annotationLiteral);
    if (_hasAnnotationLiteral) {
      ArrayList<IPluginExtension> _xblockexpression = null;
      {
        String tmp = "";
        EList<Annotation> _annotations = pattern.getAnnotations();
        for (final Annotation a : _annotations) {
          String _name = a.getName();
          boolean _matches = _name.matches("PatternUI");
          if (_matches) {
            EList<AnnotationParameter> _parameters = a.getParameters();
            for (final AnnotationParameter ap : _parameters) {
              String _name_1 = ap.getName();
              boolean _matches_1 = _name_1.matches("message");
              if (_matches_1) {
                ValueReference _value = ap.getValue();
                String _value_1 = ((StringValueImpl) _value).getValue();
                tmp = _value_1;
              }
            }
          }
        }
        final String message = tmp;
        String _databindingContributionId = this.databindingContributionId(pattern);
        final Procedure1<IPluginExtension> _function = new Procedure1<IPluginExtension>() {
            public void apply(final IPluginExtension it) {
              final Procedure1<IPluginElement> _function = new Procedure1<IPluginElement>() {
                  public void apply(final IPluginElement it) {
                    String _packageName = DatabindingGenerator.this._eMFPatternLanguageJvmModelInferrerUtil.getPackageName(pattern);
                    String _plus = (_packageName + ".");
                    String _name = pattern.getName();
                    String _firstUpper = StringExtensions.toFirstUpper(_name);
                    String _plus_1 = (_plus + _firstUpper);
                    String _plus_2 = (_plus_1 + "DatabindingAdapter");
                    exGen.contribAttribute(it, "class", _plus_2);
                    String _fullyQualifiedName = CorePatternLanguageHelper.getFullyQualifiedName(pattern);
                    exGen.contribAttribute(it, "patternName", _fullyQualifiedName);
                    exGen.contribAttribute(it, "message", message);
                    String _packageName_1 = DatabindingGenerator.this._eMFPatternLanguageJvmModelInferrerUtil.getPackageName(pattern);
                    String _plus_3 = (_packageName_1 + ".");
                    String _matcherFactoryClassName = DatabindingGenerator.this._eMFPatternLanguageJvmModelInferrerUtil.matcherFactoryClassName(pattern);
                    String _plus_4 = (_plus_3 + _matcherFactoryClassName);
                    exGen.contribAttribute(it, "matcherFactoryClass", _plus_4);
                  }
                };
              exGen.contribElement(it, "databinding", _function);
            }
          };
        IPluginExtension _contribExtension = exGen.contribExtension(_databindingContributionId, DatabindingGenerator.DATABINDINGEXTENSION_POINT, _function);
        ArrayList<IPluginExtension> _newArrayList = CollectionLiterals.<IPluginExtension>newArrayList(_contribExtension);
        _xblockexpression = (_newArrayList);
      }
      _xifexpression = _xblockexpression;
    } else {
      return CollectionLiterals.<IPluginExtension>newArrayList();
    }
    return _xifexpression;
  }
  
  public String getElementOfObservableValue(final Annotation a, final String literal) {
    EList<AnnotationParameter> _parameters = a.getParameters();
    for (final AnnotationParameter ap : _parameters) {
      String _name = ap.getName();
      boolean _matches = _name.matches(literal);
      if (_matches) {
        ValueReference _value = ap.getValue();
        return ((StringValueImpl) _value).getValue();
      }
    }
    return null;
  }
  
  public boolean hasAnnotationLiteral(final Pattern pattern, final String literal) {
    EList<Annotation> _annotations = pattern.getAnnotations();
    for (final Annotation a : _annotations) {
      String _name = a.getName();
      boolean _matches = _name.matches(literal);
      if (_matches) {
        return true;
      }
    }
    return false;
  }
  
  public CharSequence patternHandler(final Pattern pattern) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("package ");
    String _packageName = this._eMFPatternLanguageJvmModelInferrerUtil.getPackageName(pattern);
    _builder.append(_packageName, "");
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.append("import java.util.HashMap;");
    _builder.newLine();
    _builder.newLine();
    _builder.append("import org.eclipse.viatra2.emf.incquery.databinding.runtime.BaseGeneratedDatabindingAdapter;");
    _builder.newLine();
    _builder.newLine();
    _builder.append("import ");
    String _packageName_1 = this._eMFPatternLanguageJvmModelInferrerUtil.getPackageName(pattern);
    String _plus = (_packageName_1 + ".");
    String _matchClassName = this._eMFPatternLanguageJvmModelInferrerUtil.matchClassName(pattern);
    String _plus_1 = (_plus + _matchClassName);
    _builder.append(_plus_1, "");
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.append("public class ");
    String _name = pattern.getName();
    String _firstUpper = StringExtensions.toFirstUpper(_name);
    _builder.append(_firstUpper, "");
    _builder.append("DatabindingAdapter extends BaseGeneratedDatabindingAdapter<");
    String _matchClassName_1 = this._eMFPatternLanguageJvmModelInferrerUtil.matchClassName(pattern);
    _builder.append(_matchClassName_1, "");
    _builder.append("> {");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.append("\t");
    _builder.append("public ");
    String _name_1 = pattern.getName();
    String _firstUpper_1 = StringExtensions.toFirstUpper(_name_1);
    _builder.append(_firstUpper_1, "	");
    _builder.append("DatabindingAdapter() {");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t");
    _builder.append("super();");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("parameterMap = new HashMap<String, String>();");
    _builder.newLine();
    _builder.append("\t\t");
    final Map<String,String> valueMap = DatabindingAdapterUtil.calculateObservableValues(pattern);
    _builder.newLineIfNotEmpty();
    {
      Set<String> _keySet = valueMap.keySet();
      for(final String value : _keySet) {
        _builder.append("\t\t");
        _builder.append("parameterMap.put(\"");
        _builder.append(value, "		");
        _builder.append("\", \"");
        String _get = valueMap.get(value);
        _builder.append(_get, "		");
        _builder.append("\");");
        _builder.newLineIfNotEmpty();
      }
    }
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t");
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    _builder.newLine();
    return _builder;
  }
  
  public IPath[] getAdditionalBinIncludes() {
    Path _path = new Path("plugin.xml");
    return ((IPath[])Conversions.unwrapArray(CollectionLiterals.<Path>newArrayList(_path), IPath.class));
  }
}
